#pragma once
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include "matrix3.h"

typedef struct cam_response_t
{
  float *rgb;
  int lambda_min;
  int lambda_max;
  int lambda_step;

  float cam_to_xyz[9];
}
cam_response_t;

static inline cam_response_t *cam_read(const char *spectrumfile, const char *matrixfile)
{
  FILE *f = fopen(spectrumfile, "r");
  assert(f);
  cam_response_t *c = malloc(sizeof(*c));
  c->lambda_min = 400;
  c->lambda_max = 700;
  c->lambda_step = 4;
  c->rgb = malloc(sizeof(float) * 3 * ((c->lambda_max - c->lambda_min)/c->lambda_step + 1));

  int i = 0;
  while(!feof(f))
  {
    assert(i < (c->lambda_max - c->lambda_min)/c->lambda_step + 1);
    fscanf(f, "%*g %g %g %g\n", c->rgb + 3*i, c->rgb + 3*i+1, c->rgb + 3*i+2);
    i++;
  }

  fclose(f);

  f = fopen(matrixfile, "r");
  float inv[9];
  fscanf(f, "%g %g %g\n", inv+0, inv+1, inv+2);
  fscanf(f, "%g %g %g\n", inv+3, inv+4, inv+5);
  fscanf(f, "%g %g %g\n", inv+6, inv+7, inv+8);
  mat3_invert(inv, c->cam_to_xyz);
  // fprintf(stderr, "matrix: %g %g %g\n %g %g %g\n %g %g %g\n",
  //     c->cam_to_xyz[0], c->cam_to_xyz[1], c->cam_to_xyz[2],
  //     c->cam_to_xyz[3], c->cam_to_xyz[4], c->cam_to_xyz[5],
  //     c->cam_to_xyz[6], c->cam_to_xyz[7], c->cam_to_xyz[8]);
  fclose(f);
  return c;
}

static inline void cam_response(const cam_response_t *c, float lambda, float *rgb)
{
  if(lambda < c->lambda_min || lambda >= c->lambda_max) { rgb[0] = rgb[1] = rgb[2] = 0.0f; return; }
  float f = (lambda - c->lambda_min)/c->lambda_step;
  int i = (int)f;
  f -= i;
  rgb[0] = (1-f)*c->rgb[3*i+0] + f*c->rgb[3*(i+1)+0];
  rgb[1] = (1-f)*c->rgb[3*i+1] + f*c->rgb[3*(i+1)+1];
  rgb[2] = (1-f)*c->rgb[3*i+2] + f*c->rgb[3*(i+1)+2];
}

static inline void cam_rgb_to_xyz(const cam_response_t *c, const float *cam_rgb, float *xyz)
{
  mat3_mulv(c->cam_to_xyz, cam_rgb, xyz);
}
